@@ -18,15 +18,16 @@
 
 import json
 import logging
+import os
 import re
 from base64 import b64encode
-from typing import Callable, Optional
+from typing import Callable, Dict, Optional
 
 import hy
 import requests
 from bs4 import BeautifulSoup
 
-from raider.utils import hy_dict_to_python, match_tag
+from raider.utils import hy_dict_to_python, match_tag, parse_json_filter
 
 
 class Plugin:
@@ -90,10 +91,16 @@ class Plugin:
 
         """
         self.name = name
-        self.function = function
         self.value: Optional[str] = value
         self.flags = flags
 
+        self.function: Callable[..., Optional[str]]
+
+        if flags & Plugin.NEEDS_USERDATA:
+            self.function = self.extract_from_userdata
+        else:
+            self.function = function
+
     def extract_value(
         self,
         response: Optional[requests.models.Response],
@@ -123,7 +130,7 @@ class Plugin:
 
     def get_value(
         self,
-        userdata: dict[str, str],
+        userdata: Dict[str, str],
     ) -> Optional[str]:
         """Gets the value from the Plugin.
 
@@ -134,9 +141,38 @@ class Plugin:
             A dictionary with the user specific data.
         """
         if self.needs_userdata:
-            self.function(userdata)
+            self.value = self.function(userdata)
         elif not self.needs_response:
-            self.function()
+            self.value = self.function()
+        return self.value
+
+    def extract_from_userdata(
+        self, data: Dict[str, str] = None
+    ) -> Optional[str]:
+        """Extracts the plugin value from userdata.
+
+        Given a dictionary with the userdata, return its value with the
+        same name as the "name" attribute from this Plugin.
+
+        Args:
+          data:
+            A dictionary with user specific data.
+
+        Returns:
+          A string with the value of the variable found. None if no such
+          variable has been defined.
+
+        """
+        if data and self.name in data:
+            self.value = data[self.name]
+        return self.value
+
+    def return_value(self) -> Optional[str]:
+        """Just return plugin's value.
+
+        This is used when needing a function just to return the value.
+
+        """
         return self.value
 
     @property
@@ -195,7 +231,7 @@ class Regex(Plugin):
         super().__init__(
             name=name,
             function=self.extract_regex,
-            flags=self.NEEDS_RESPONSE,
+            flags=Plugin.NEEDS_RESPONSE,
         )
         self.regex = regex
         self.extract = extract
@@ -259,7 +295,7 @@ class Html(Plugin):
         self,
         name: str,
         tag: str,
-        attributes: dict[hy.HyKeyword, str],
+        attributes: Dict[hy.HyKeyword, str],
         extract: str,
     ) -> None:
         """Initializes the Html Plugin.
@@ -285,7 +321,7 @@ class Html(Plugin):
         super().__init__(
             name=name,
             function=self.extract_html_tag,
-            flags=self.NEEDS_RESPONSE,
+            flags=Plugin.NEEDS_RESPONSE,
         )
         self.tag = tag
         self.attributes = hy_dict_to_python(attributes)
@@ -334,9 +370,20 @@ class Html(Plugin):
 class Json(Plugin):
     """Plugin to extract a field from JSON.
 
-    The "extract" attribute is used to specify which field to store
-    in the "value". Using the dot "." character you can go deeper inside
-    the JSON object. No arrays are supported yet.
+    The "extract" attribute is used to specify which field to store in
+    the "value". Using the dot ``.`` character you can go deeper inside
+    the JSON object. To look inside an array, use square brackets
+    `[]`.
+
+    Keys with special characters should be written inside double quotes
+    ``"``. Keep in mind that when written inside ``hyfiles``,
+    it'll already be between double quotes, so you'll have to escape
+    them with the backslash character ``\\``.
+
+    Examples:
+
+      ``env.production[0].field``
+      ``production.keys[1].x5c[0][1][0]."with space"[3]``
 
     Attributes:
       extract:
@@ -360,7 +407,7 @@ class Json(Plugin):
         super().__init__(
             name=name,
             function=self.extract_json_field,
-            flags=self.NEEDS_RESPONSE,
+            flags=Plugin.NEEDS_RESPONSE,
         )
         self.extract = extract
 
@@ -381,15 +428,45 @@ class Json(Plugin):
           found None will be returned.
 
         """
-        parsed = json.loads(response.text)
+        data = json.loads(response.text)
 
-        items = self.extract.split(".")
-        temp = parsed
-        for item in items:
-            temp = temp[item]
+        json_filter = parse_json_filter(self.extract)
+        is_valid = True
+        temp = data
+        for item in json_filter:
+            if item.startswith("["):
+                index = int(item.strip("[]"))
+                if len(temp) > index:
+                    temp = temp[index]
+                else:
+                    logging.warning(
+                        (
+                            "JSON array index doesn't exist.",
+                            "Cannot extract plugin's value.",
+                        )
+                    )
+                    is_valid = False
+                    break
+            else:
+                if item in temp:
+                    temp = temp[item]
+                else:
+                    logging.warning(
+                        (
+                            "Key '%s' not found in the response body.",
+                            "Cannot extract plugin's value.",
+                        ),
+                        item,
+                    )
+                    is_valid = False
+                    break
+
+        if is_valid:
+            self.value = str(temp)
+            logging.debug("Json filter %s: %s", self.name, str(self.value))
+        else:
+            self.value = None
 
-        self.value = str(temp)
-        logging.debug("Json filter %s: %s", self.name, str(self.value))
         return self.value
 
     def __str__(self) -> str:
@@ -417,12 +494,36 @@ class Variable(Plugin):
         """
         super().__init__(
             name=name,
-            function=self.extract_variable,
-            flags=self.NEEDS_USERDATA,
+            function=lambda: self.value,
+            flags=Plugin.NEEDS_USERDATA,
         )
 
-    def extract_variable(self, data: dict[str, str] = None) -> Optional[str]:
-        """Extracts the variable value.
+
+class Command(Plugin):
+    """Runs a shell command and extract the output."""
+
+    def __init__(self, name: str, command: str) -> None:
+        """Initializes the Command Plugin.
+
+        The specified command will be executed with os.popen() and the
+        output with the stripped last newline, will be saved inside the
+        value.
+
+        Args:
+          name:
+            A unique identifier for the plugin.
+          command:
+            The command to be executed.
+
+        """
+        self.command = command
+        super().__init__(
+            name=name,
+            function=self.run_command,
+        )
+
+    def run_command(self) -> Optional[str]:
+        """Runs a command and returns its value.
 
         Given a dictionary with the predefined variables, return the
         value of the with the same name as the "name" attribute from
@@ -437,8 +538,8 @@ class Variable(Plugin):
           variable has been defined.
 
         """
-        if data and self.name in data:
-            self.value = data[self.name]
+        self.value = os.popen(self.command).read().strip()
+
         return self.value
 
 
@@ -488,6 +589,7 @@ class Cookie(Plugin):
         name: str,
         value: Optional[str] = None,
         function: Optional[Callable[..., Optional[str]]] = None,
+        flags: int = Plugin.NEEDS_RESPONSE,
     ) -> None:
         """Initializes the Cookie Plugin.
 
@@ -507,16 +609,27 @@ class Cookie(Plugin):
 
         """
         if not function:
-            super().__init__(
-                name=name,
-                function=self.extract_cookie,
-                value=value,
-                flags=self.NEEDS_RESPONSE,
-            )
-        else:
-            super().__init__(name, function)
+            if flags & Plugin.NEEDS_RESPONSE:
+                super().__init__(
+                    name=name,
+                    function=self.extract_from_response,
+                    value=value,
+                    flags=flags,
+                )
+            else:
+                super().__init__(
+                    name=name,
+                    function=lambda: self.value,
+                    value=value,
+                    flags=flags,
+                )
 
-    def extract_cookie(
+        else:
+            super().__init__(
+                name=name, function=function, value=value, flags=flags
+            )
+
+    def extract_from_response(
         self, response: requests.models.Response
     ) -> Optional[str]:
         """Returns the cookie with the specified name from the response."""
@@ -526,6 +639,31 @@ class Cookie(Plugin):
         """Returns a string representation of the cookie."""
         return str({self.name: self.value})
 
+    @classmethod
+    def from_plugin(cls, name: str, plugin: Plugin) -> "Cookie":
+        """Creates a Cookie from a Plugin.
+
+        Given another :class:`plugin <raider.plugins.Plugin>`, and a
+        name, create a :class:`cookie <raider.plugins.Cookie>`.
+
+        Args:
+          name:
+            The cookie name to use.
+          plugin:
+            The plugin which will contain the value we need.
+
+        Returns:
+          A Cookie object with the name and the plugin's value.
+
+        """
+        cookie = cls(
+            name=name,
+            value=None,
+            function=lambda: plugin.value if plugin.value else None,
+            flags=0,
+        )
+        return cookie
+
 
 class Header(Plugin):
     """Plugin to deal with HTTP headers.
@@ -538,6 +676,7 @@ class Header(Plugin):
         name: str,
         value: Optional[str] = None,
         function: Optional[Callable[..., Optional[str]]] = None,
+        flags: int = Plugin.NEEDS_RESPONSE,
     ) -> None:
         """Initializes the Header Plugin.
 
@@ -558,18 +697,28 @@ class Header(Plugin):
         """
 
         if not function:
-            super().__init__(
-                name=name,
-                function=lambda: self.value,
-                value=value,
-                flags=self.NEEDS_RESPONSE,
-            )
+            if flags & Plugin.NEEDS_RESPONSE:
+                super().__init__(
+                    name=name,
+                    function=self.extract_from_response,
+                    value=value,
+                    flags=flags,
+                )
+
+            else:
+                super().__init__(
+                    name=name,
+                    function=lambda: self.value,
+                    value=value,
+                    flags=flags,
+                )
+
         else:
             super().__init__(
-                name=name, function=function, flags=self.NEEDS_RESPONSE
+                name=name, function=function, value=value, flags=flags
             )
 
-    def extract_header(
+    def extract_from_response(
         self, response: requests.models.Response
     ) -> Optional[str]:
         """Returns the header with the specified name from the response."""
@@ -617,10 +766,110 @@ class Header(Plugin):
 
         """
         header = cls(
-            "Authorization",
-            None,
-            lambda: "Bearer " + access_token.value
+            name="Authorization",
+            value=None,
+            flags=0,
+            function=lambda: "Bearer " + access_token.value
             if access_token.value
             else None,
         )
         return header
+
+    @classmethod
+    def from_plugin(cls, name: str, plugin: Plugin) -> "Header":
+        """Creates a Header from a Plugin.
+
+        Given another :class:`plugin <raider.plugins.Plugin>`, and a
+        name, create a :class:`header <raider.plugins.Header>`.
+
+        Args:
+          name:
+            The header name to use.
+          plugin:
+            The plugin which will contain the value we need.
+
+        Returns:
+          A Header object with the name and the plugin's value.
+
+        """
+        header = cls(
+            name=name,
+            value=None,
+            function=lambda: plugin.value if plugin.value else None,
+            flags=0,
+        )
+        return header
+
+
+class Alter(Plugin):
+    """Plugin used to alter other plugin's value.
+
+    If the value extracted from other plugins cannot be used in it's raw
+    form and needs to be somehow processed, Alter plugin can be used to
+    do that. Initialize it with the original plugin and a function which
+    will process the string and return the modified value.
+
+    Attributes:
+      alter_function:
+        A function which will be given the plugin's value. It should
+        return a string with the processed value.
+
+    """
+
+    def __init__(
+        self, plugin: Plugin, alter_function: Callable[[str], Optional[str]]
+    ) -> None:
+        """Initializes the Alter Plugin.
+
+        Given the original plugin, and a function to alter the data,
+        initialize the object, and get the modified value.
+
+        Args:
+          plugin:
+            The original Plugin where the value is to be found.
+          alter_function:
+            The Function with instructions on how to alter the value.
+        """
+        self.alter_function = alter_function
+        super().__init__(
+            name=plugin.name,
+            value=plugin.value,
+            # Get plugin's value from userdata since it has
+            # already been extracted from the original plugin.
+            flags=Plugin.NEEDS_USERDATA,
+            function=self.process_value,
+        )
+
+    def process_value(self, data: Dict[str, str] = None) -> Optional[str]:
+        """Process the original plugin's value.
+
+        Get the value of the original's plugin from userdata, and give
+        it to ``alter_function``. Return the processed value and store
+        it in self.value.
+
+        Args:
+          data:
+            A dictionary with the userdata where the original plugin's
+            value should be found.
+
+        Returns:
+          A string with the processed value.
+        """
+        if data and self.name in data:
+            self.value = self.alter_function(data[self.name])
+
+        return self.value
+
+    @classmethod
+    def prepend(cls, plugin: Plugin, string: str) -> "Alter":
+        """Prepend a string to plugin's value."""
+        alter = cls(plugin=plugin, alter_function=lambda value: string + value)
+
+        return alter
+
+    @classmethod
+    def append(cls, plugin: Plugin, string: str) -> "Alter":
+        """Append a string after the plugin's value"""
+        alter = cls(plugin=plugin, alter_function=lambda value: value + string)
+
+        return alter
